/*********************************************************************
*	AFEDRI SDR-Net Commander utility
*	Author: Alexander Trushkin ( 4Z5LV )
*	Date: 6.11.2013
*	
*	This utility allows you to control AFEDRI SDR 
*	over USB connection.
*	So you can use this utility in command shell scripts or WebSDR
*	or in crone command   
**********************************************************************/

//#include "stdafx.h"
 #ifdef _WIN32
	#include "hidapi.h"
    #include <windows.h>
 #else
	#ifndef _GNU_SOURCE
	    #define _GNU_SOURCE
	#endif
	#include "hidapi.h"
	#include <unistd.h>
	#include <sys/types.h>	
	#include <sys/stat.h>
	#include <fcntl.h>
	#include <errno.h>
 #endif
#include "fe_interface.h"
#include <stdio.h>
#include <malloc.h>

#include <wchar.h>
#include <string.h>
#include <stdlib.h>
#include <math.h>
#include "eeprom.h"
//#include "main.h"
#include "tcp_shell.h"
//#define _PRINT_DEBUG_MESSAGES_
//#define NON_BLOCKING_HID_IO
#define RF_GAIN_ZERO    -10
#define MAX_STR 255
	int GainIndex[7]={0,4,2,6,1,5,3};
	double GainTable[7]={1.00, 1.14, 1.33, 1.60, 2.00, 2.67, 4};
        int FE_Gain = 0;
	volatile int hid_status=0;
        wchar_t wstr[MAX_STR+1];
        hid_device *fe_handle=NULL;
 #ifdef _WIN32
        HANDLE msd_handle=NULL;
        HANDLE CommOut=NULL;
 #elif __APPLE__
        FILE *msd_handle=NULL;
        FILE *CommOut=NULL;
 #else
        int msd_handle=0;
        int CommOut=0;
 #endif
        int HID_COMMAND_SIZE = 7;
        int global_device_PID = HID_DEVICE_SDR_NET_PID;
        unsigned char hid_tcp_packet[32]={0};
        volatile int hid_tcp_network_flag = 0;
        volatile int hid_command_send_state = 0;
        char Gpath_name[256]={0};
        wchar_t USB_Serial_number[256]=L"\0";
        wchar_t *SerialNumber = (wchar_t *)NULL;
 #ifdef __linux__
	char *default_path_name = "/run/media/alex/disk/";
 #elif __APPLE__
    char *default_path_name = "/Volumes/Untitled/";
 #else
        char *default_path_name = (char*)"G:\\";
 #endif

	int HID_Device_Init_HID( char * msg);
	int HID_Device_Init_MSD( char * msg, const char *path_name);

	int SetMSDPath(const char *p_name)
	{
                memset(Gpath_name, 0, sizeof(Gpath_name));
//                printf("Path: %s\n\r", p_name);
                strcpy(Gpath_name, p_name);
		return 0;
	}

	void MSD_Envy_Init()
	{
		memset(Gpath_name, 0, sizeof(Gpath_name));
	}

	int Set_USB_Serial_number(wchar_t *serial)
	{
		if(serial == NULL) return -1;
		SerialNumber = wcscpy(USB_Serial_number, serial);
		return 0;
	}

	int HID_Device_Enumerate()
	{
		struct hid_device_info *devs = 0;
		devs = hid_enumerate(HID_DEVICE_VID, HID_DEVICE_PID);
		if(devs != 0)
		{
			hid_free_enumeration(devs);
			return 1;
		}
		else
		{
                    devs = hid_enumerate(HID_DEVICE_ATEECS_VID, HID_DEVICE_SDR_NET_PID);
                    if(devs != 0)
                    {
                            hid_free_enumeration(devs);
                            return 1;
                    }
                    else
                    {
                        devs = hid_enumerate(HID_DEVICE_ATEECS_VID, HID_DEVICE_ATEECS_PID);
                        if(devs != 0)
                        {
                                hid_free_enumeration(devs);
                                return 1;
                        }
                        else
                        {
                            return 0;
                        }
                    }
		}

	}

	int HID_Device_Reconnect( char * msg)
	{
		int res=0;
                if(hid_status)
		{
                  HID_Close();
		}
		res = HID_Device_Init(msg);
		return res;
	}

        int HID_Close()
	{
		if(fe_handle)  hid_close(fe_handle);
   //             if(msd_handle != INVALID_HANDLE_VALUE) CloseHandle(msd_handle);
   //             if(CommOut != INVALID_HANDLE_VALUE) CloseHandle(CommOut);
                return 0;
	}

	int HID_Device_Init( char * msg)
	{
		int res = 0;
		res = HID_Device_Init_HID( msg);
#ifdef _MSD_INTERFACE_
                if(res == 0)
		{ 
                //	printf("HID: %s\n\r", msg);
                        if(strlen(Gpath_name) == 0)
		 	{
                                printf("The AFEDRI path was not defined, use default: %s\n\r",default_path_name);
                                res = HID_Device_Init_MSD( msg, (const char*)default_path_name);
			}
			else
			{
                                printf("The AFEDRI path is: %s\n\r",Gpath_name);
                                res = HID_Device_Init_MSD( msg, (const char*) Gpath_name);
                        }

		}
#endif
                return res;
	}
	
	int HID_Device_Init_HID( char * msg)
	{
        int res;
        fe_handle = hid_open(HID_DEVICE_VID, HID_DEVICE_PID, SerialNumber);
        if (!fe_handle)
        {
            fe_handle = hid_open(HID_DEVICE_ATEECS_VID, HID_DEVICE_SDR_NET_PID, SerialNumber);
            if (!fe_handle)
            {
                fe_handle = hid_open(HID_DEVICE_ATEECS_VID, HID_DEVICE_ATEECS_PID, SerialNumber);
                if (!fe_handle)
                {
                    fe_handle = hid_open(HID_DEVICE_ATEECS_VID, HID_DEVICE_ATEECS_PID_6, SerialNumber);
                    if (!fe_handle)
                    {
                        strcpy(msg ,"Unable to open device!");
                        hid_status=0;
                        return 0;
                    }
                    else
                    {
                        global_device_PID = HID_DEVICE_SDR_NET_PID;
                        HID_COMMAND_SIZE = 7;
                    }
                }
                else
                {
                    global_device_PID = HID_DEVICE_ATEECS_PID;
                    HID_COMMAND_SIZE = 5;
                }
            }
            else
            {
                global_device_PID = HID_DEVICE_SDR_NET_PID;
                HID_COMMAND_SIZE = 7;
            }
        }
        else
        {
            global_device_PID = HID_DEVICE_ATEECS_PID;
            HID_COMMAND_SIZE = 5;
        }
        if(fe_handle)
        {
            hid_status=1;
            wstr[0] = 0x0000;
            hid_set_nonblocking(fe_handle, 1);
            res = hid_get_product_string(fe_handle, wstr, MAX_STR);
            if (res < 0)
            {
                strcpy(msg,"Unable to read product string");
            }
            else
            {
                wcstombs(msg, wstr, wcslen(wstr));
            }
                   // Set the hid_read() function to be non-blocking.
#ifdef NON_BLOCKING_HID_IO
            res = hid_set_nonblocking(fe_handle, 1);
#else
            res = hid_set_nonblocking(fe_handle, 0);
#endif
        //                  if(res < 0) return res;

        }
        return 1;
	}

#ifdef _MSD_INTERFACE_
    int HID_Device_Init_MSD( char * msg, const char *path_name)
	{
//		int res;
                char in_name[256];
                char out_name[256];
                char *tmp_name;
        #ifndef __linux__
//                const size_t cSize;
		#ifndef _COMMAND_SHELL_
                	wchar_t *pwPath;
	        #endif
        #endif
                //strcpy(msg ,"Test error!");
//                hid_status=0;
//                msd_handle = (HANDLE)-1;
//                return 0;
                memset(in_name, 0 , sizeof(in_name));
                memset(out_name, 0 , sizeof(out_name));
                strcpy(in_name,path_name);
                strcpy(out_name,path_name);
                tmp_name = strcat(in_name, "command.in");
	#ifdef __linux__
                msd_handle = open(tmp_name, O_DIRECT | O_SYNC | O_RDWR);
              // msd_handle = CreateFile(tmp_name,GENERIC_ALL,0,NULL,OPEN_EXISTING,FILE_FLAG_NO_BUFFERING | FILE_FLAG_WRITE_THROUGH, NULL);
    #elif __APPLE__
                msd_handle = fopen(tmp_name,  "r+");
                //fcntl(msd_handle, F_NOCACHE, 1);
                setvbuf(msd_handle, (char *)NULL, _IONBF, 0);
        // msd_handle = CreateFile(tmp_name,GENERIC_ALL,0,NULL,OPEN_EXISTING,FILE_FLAG_NO_BUFFERING | FILE_FLAG_WRITE_THROUGH, NULL);
	#else
	   #ifdef _COMMAND_SHELL_
                msd_handle = CreateFile(tmp_name,GENERIC_ALL,0,NULL,OPEN_EXISTING,FILE_FLAG_NO_BUFFERING | FILE_FLAG_WRITE_THROUGH, NULL);
	   #else
                pwPath = (wchar_t*)malloc(strlen(tmp_name)*2+1);
                if(!pwPath)
                {
                    printf("(input) Cannot allocate wchar_t space!\n\r");
                    return -1;
                }
                mbstowcs(pwPath, tmp_name, strlen(tmp_name)+1);
                msd_handle = CreateFile(pwPath,GENERIC_ALL,0,NULL,OPEN_EXISTING,FILE_FLAG_NO_BUFFERING | FILE_FLAG_WRITE_THROUGH, NULL);
                free(pwPath);
	   #endif
	#endif
                //hid_open(HID_DEVICE_VID, HID_DEVICE_PID, NULL);
        #if defined ( __linux__ )
                if (msd_handle < 0)
        #elif defined (__APPLE__)
                if (msd_handle == NULL)
        #else
                if (msd_handle == INVALID_HANDLE_VALUE)
        #endif
                {
                  strcpy(msg ,"Unable to open Input device!\r\n");
 			#ifdef _WIN32
                 fprintf(stderr, "Input file %s open  error %li\n\r",tmp_name, GetLastError());
			#endif
                  hid_status=0;
                  return 0;
                }
                tmp_name = strcat(out_name, "command.out");
	#ifdef __linux__
                CommOut = open(tmp_name, O_DIRECT | O_SYNC | O_RDWR);
//#if defined ( __linux__ ) || defined (__APPLE__)
    #elif __APPLE__
                CommOut = fopen(tmp_name,  "r+");
                setvbuf(CommOut, (char *)NULL, _IONBF, 0);
//              	CommOut = open(tmp_name,  O_SYNC | O_RDWR);
//                fcntl(CommOut, F_NOCACHE, 1);
	#else
	   #ifdef _COMMAND_SHELL_
                CommOut = CreateFile(tmp_name,GENERIC_ALL,0,NULL,OPEN_EXISTING,FILE_FLAG_NO_BUFFERING | FILE_FLAG_WRITE_THROUGH, NULL);
	   #else
                pwPath = (wchar_t*)malloc(strlen(tmp_name)*2+1);
                if(!pwPath)
                {
                    printf("(output) Cannot allocate wchar_t space!\n\r");
                    return -1;
                }
                mbstowcs(pwPath, tmp_name, strlen(tmp_name)+1);
                CommOut = CreateFile(pwPath,GENERIC_ALL,0,NULL,OPEN_EXISTING,FILE_FLAG_NO_BUFFERING | FILE_FLAG_WRITE_THROUGH, NULL);
                free(pwPath);
	   #endif
        #endif
    #if defined ( __linux__ )
        if (CommOut < 0)
    #elif defined (__APPLE__)
        if (CommOut == NULL)
	#else
                 if(CommOut == INVALID_HANDLE_VALUE)
	#endif
                 {
        #if defined ( __linux__ )
                     close(CommOut);
        #elif  defined (__APPLE__)
                     fclose(CommOut);
		#else
                      CloseHandle(msd_handle);
		#endif
                      strcpy(msg ,"Unable to open Output device!\r\n");
                      hid_status=0;
		#ifdef _WIN32
                      fprintf(stderr,"Output file error %li\n\r",GetLastError());
		#endif
//                      printf("%s\n\r",msg);
                      return 0;
                 }
                else
                {
                    global_device_PID = HID_DEVICE_SDR_NET_PID;
                    HID_COMMAND_SIZE = 7;
                    strcpy(msg,"AFEDRI MSD Device");
                    printf("%s\n",msg);
                    hid_status=2;
                }
//                printf("%s \n\r",msg);
                return 1;
	}
#endif
	int HID_Read_Data( char *buf)
	{
                int i,res=0;
		long result=0;
		char *dummy;//[MSD_CLUSTER_SIZE] = {0};
#if defined ( __linux__ )
                char message[64] = {0};
                char * err_s;
#elif defined ( __APPLE__ )
            char message[64] = {0};
            char * err_s;
#endif

                buf[0] = 0;
//                printf("HID data: ");
//                for(int i=0; i<HID_COMMAND_SIZE;i++) printf("%2.2X",buf[i]);
//                printf("\n\r");
                if(hid_tcp_network_flag == 1)
                {
                    if(((hid_tcp_packet[1] >> 5) == TCP_HID_PACKET) && (hid_tcp_packet[0] != 0))
                    {
                        for( i=0; i < HID_COMMAND_SIZE; i++)
                        {
                            buf[i] = hid_tcp_packet[i+2];
                        }
                        hid_tcp_packet[0] = 0;
                        return HID_COMMAND_SIZE;
                    }
                    else return 0;
                }
                else if(hid_status == 1 && fe_handle != 0)
                {
//                        res = hid_set_nonblocking(fe_handle, 1);
                        // Try to read from the device. There shoud be no
                        // data here, but execution should not block.
#ifndef NON_BLOCKING_HID_IO
            res = hid_set_nonblocking(fe_handle, 0);
#endif
//                        res = hid_read(fe_handle, (unsigned char *)buf, HID_COMMAND_SIZE);
                        res = hid_read_timeout(fe_handle, (unsigned char *)buf, HID_COMMAND_SIZE, 1000);
                        if(res >= 5) hid_command_send_state = 0;
                        else res = -1;
#ifdef _PRINT_DEBUG_MESSAGES_
                        printf("HID packet: ");
                        for(int k=0; k<res;k++)
                        {
                            printf("%2.2X ",(unsigned char)buf[k]);
                        }
                        printf("\n\r");
#endif
                        return res;
                }
#ifdef _MSD_INTERFACE_
    #if defined ( __linux__ ) 
                else if(hid_status == 2)
                {
                    dummy = (char *) memalign(MSD_CLUSTER_SIZE, MSD_CLUSTER_SIZE);
    #elif __APPLE__
                else if(hid_status == 2)
                {
                    posix_memalign(&dummy, MSD_CLUSTER_SIZE, MSD_CLUSTER_SIZE);
	#else
                else if(hid_status == 2 && msd_handle != INVALID_HANDLE_VALUE)
                {
                    dummy = (char *) __mingw_aligned_malloc(MSD_CLUSTER_SIZE, MSD_CLUSTER_SIZE);
	#endif
                    if(!dummy)
                    {
                        printf("Cannot allocate buffer!\r\n");
                        return -1;
                    }
                    memset(dummy, 0, MSD_CLUSTER_SIZE);
        #ifndef NON_BLOCKING_HID_IO
                    while(dummy[0] == 0)
        #endif
                    {
    #if defined ( __linux__ )
                        lseek(msd_handle, 0, SEEK_SET);
                        res = read(msd_handle,dummy, MSD_CLUSTER_SIZE);
    #elif defined (__APPLE__)
                        rewind(msd_handle);
                        fpurge(msd_handle);
                        res = fread(dummy, 1, MSD_CLUSTER_SIZE, msd_handle);
	#else
                        SetFilePointer(msd_handle,0,NULL,0);
                        res = ReadFile(msd_handle,dummy,MSD_CLUSTER_SIZE, (DWORD*)&result, NULL);//&overlapped_structure);
	#endif
                        if((res > 0) && (dummy[0] != 0))
                        {
                            memcpy(buf, dummy, HID_COMMAND_SIZE);
	#if defined ( __linux__ )
                            lseek(msd_handle, 0, SEEK_SET);
                            dummy[0] = 0;
                            res = write(msd_handle,dummy, MSD_CLUSTER_SIZE);
                            if (res < 0)
                            {
                                err_s = strerror_r(errno, message, 64);
                            }
                            free(dummy);
    #elif  defined (__APPLE__)
                            rewind(msd_handle);
                            dummy[0] = 0;
                            res = fwrite(dummy, 1, MSD_CLUSTER_SIZE, msd_handle);
                            if (res == 0)
                            {
                                err_s = strerror_r(errno, message, 64);
                            }
                            fflush(msd_handle);
                            free(dummy);
	#else
                            SetFilePointer(msd_handle,0,NULL,0);
                            dummy[0] = 0;
                            res = WriteFile(msd_handle,dummy,MSD_CLUSTER_SIZE, (DWORD*)&result, NULL);//&overlapped_structure);
                            __mingw_aligned_free(dummy);

        #endif
                            hid_command_send_state = 0;
                            return res;
                        }
                        else
                        {
	#if defined ( __linux__ ) || defined (__APPLE__)
  //                              printf("Read error: %i \n", res);
	#else
//                                printf("Reading error: %li \n", GetLastError());
	#endif
                                res = 0;
                        }
#ifndef NON_BLOCKING_HID_IO
    #if defined ( __linux__ ) || defined (__APPLE__)
                        sleep(1);
 	#else
                        Sleep(1);
 	#endif
#endif
                    }
                    return res;
                }
#endif
                return 0;
          }

        int HID_Write_Data(unsigned char *report)
        {
            int  i,res=0;
            long result=0;
            unsigned char buffer[16]={0};
            char *tmp_buf;
#ifdef _PRINT_DEBUG_MESSAGES_
            printf("HID write data: ");
            for(int i=0; i<HID_COMMAND_SIZE;i++) printf("%2.2X ",report[i]);
            printf("\n\r");
#endif
            if(hid_tcp_network_flag == 1)
            {
                buffer[0] = HID_COMMAND_SIZE + 2;
                buffer[1] = TCP_HID_PACKET << 5;
                for(i=0; i < HID_COMMAND_SIZE; i++)
                {
                    buffer[i+2] = report[i];
                }
                tcp_send_hid_command((char *)buffer, HID_COMMAND_SIZE+2);
                hid_command_send_state = 1;
                return 1;
            }
            else if(hid_status == 1)
            {
                res = hid_write(fe_handle, (const unsigned char *)report, HID_COMMAND_SIZE);
                if(res > 0) hid_command_send_state = 1;
                else hid_command_send_state = 0;
                return res;
            }
#ifdef _MSD_INTERFACE_
	#ifdef __linux__
            else  if(hid_status == 2)
            {
               tmp_buf = memalign(MSD_CLUSTER_SIZE, MSD_CLUSTER_SIZE);
    #elif __APPLE__
                else  if(hid_status == 2)
                {
                     posix_memalign(&tmp_buf, MSD_CLUSTER_SIZE, MSD_CLUSTER_SIZE);
	#else
            else  if(hid_status == 2 && CommOut != INVALID_HANDLE_VALUE)
            {
                tmp_buf = (char *) __mingw_aligned_malloc(MSD_CLUSTER_SIZE, MSD_CLUSTER_SIZE);
	#endif
                if(!tmp_buf)
                {
                        printf("Cannot allocate buffer!\r\n");
                        return -1;
                }
                for( i=0; i<HID_COMMAND_SIZE; i++)
                {
                    tmp_buf[i] = report[i];
                }
	#if defined ( __linux__ )
                lseek(CommOut, 0, SEEK_SET);
                res = write(CommOut,tmp_buf, MSD_CLUSTER_SIZE);
                free(tmp_buf);
    #elif   defined (__APPLE__)
                rewind(CommOut);
                res = fwrite(tmp_buf, 1, MSD_CLUSTER_SIZE, CommOut);
                if (res == 0)
                {
                    //err_s = strerror_r(errno, message, 64);
                }
                fflush(CommOut);
                free(tmp_buf);
	#else
                SetFilePointer(CommOut,0,NULL,0);
                res = WriteFile(CommOut,tmp_buf,MSD_CLUSTER_SIZE, (DWORD*)&result, NULL);//&overlapped_structure);
                __mingw_aligned_free(tmp_buf);
	#endif
                if(res > 0) hid_command_send_state = 1;
                else hid_command_send_state = 0;
                return res;
            }
#endif
            return 0;
        }

        void hid_change_tcp_network_flag(int state)
        {
            hid_tcp_network_flag = state;
        }  

        int hid_save_recieved_tcp_packet(char *data, int size)
        {
            int i;
            if(size > 32) size = 32;
            for(i=0; i < size; i++)
            {
                if(data != NULL) hid_tcp_packet[i] = data[i];
            }
            return 0;
        }

        int HID_Set_Frequency(unsigned long frequency, int channel)
        {
                int i;
                unsigned char report[HID_COMMAND_MAX_SIZE]={0};
#ifdef _PRINT_DEBUG_MESSAGES_
		printf("HID FREQ: %lu, ch=%i (%i)\n\r",frequency, channel,HID_COMMAND_MAX_SIZE);
#endif
		report[0] = HID_FREQUENCY_REPORT;
		for(i=0; i < 4;  i++)
		{
			report[i+1]= (frequency >> (i*8)) & 0xFF;
		}
                report[5] = channel;
                for(i=6; i < HID_COMMAND_MAX_SIZE;  i++) report[i]=0;
                return HID_Write_Data((unsigned char *)report);
	}

        int HID_Get_Frequency(int channel)
	{
                int i;
                char report[HID_COMMAND_MAX_SIZE]={0};
                report[0] = HID_GENERIC_REPORT;
                report[1]=  HID_GENERIC_GET_FREQ_COMMAND & 0xFF;
                report[2]= channel;
                for(i=3; i < HID_COMMAND_MAX_SIZE;  i++) report[i]=0;
                return HID_Write_Data((unsigned char *)report);
	}

	unsigned long HID_Read_Frequency(const unsigned char *buf)
	{
		unsigned long frequency =0;
		unsigned long long_tmp;
		int i;
		for(i =0; i<4; i++)
		{
			long_tmp = buf[i+1] & 0xFF;
			frequency += long_tmp << (i*8);
		}
		return frequency;
	}

        unsigned long HID_Get_Generic_Report(unsigned char *generic_command, const unsigned char *buf)
	{
		unsigned long data =0;
		unsigned long long_tmp;
		int i;
                *generic_command = buf[1];
                for(i =0; i<4; i++)
		{
                        long_tmp = buf[i+2] & 0xFF;
			data += long_tmp << (i*8);
		}
		return data;
	}

        int HID_ChangeFE_Gain(int index, int fe_handle)
	{
                int i;
                char report[HID_COMMAND_MAX_SIZE]={0};
                report[0] = HID_GENERIC_REPORT;
                report[1]=  HID_GENERIC_GAIN_COMMAND & 0xFF;
                report[2]= index+1;
                report[3] = fe_handle;
                for( i=4; i < HID_COMMAND_MAX_SIZE;  i++) report[i]=0;
                return HID_Write_Data((unsigned char *)report);
	}

        int HID_ChangeRF_Gain(int channel, int dac_value)
	{
                int i;
                char report[HID_COMMAND_MAX_SIZE]={0};
			report[0] = HID_GENERIC_REPORT;
			report[1]=  HID_GENERIC_DAC_COMMAND & 0xFF;
			report[2]= dac_value & 0xFF;
                        report[3]= channel & 0xFF;
                        for( i=4; i < HID_COMMAND_MAX_SIZE;  i++) report[i]=0;
                        return HID_Write_Data((unsigned char *)report);
	}

        int HID_Change_VCO_voltage(int vco_value)
        {
                int i;
                short tmp_vco = (2048 + vco_value) & 0xFFF;
                char report[HID_COMMAND_MAX_SIZE]={0};
                        report[0] = HID_GENERIC_REPORT;
                        report[1]=  HID_GENERIC_SET_VCO_COMMAND & 0xFF;
                        report[2]= tmp_vco & 0xFF;
                        report[3]= (tmp_vco >> 8) & 0xFF;
                        for( i=4; i < HID_COMMAND_MAX_SIZE;  i++) report[i]=0;
                        return HID_Write_Data((unsigned char *)report);
        }

        int HID_Set_Overload_Mode(int mode)
        {
                unsigned short i;
                char report[HID_COMMAND_MAX_SIZE]={0};
                        report[0] = HID_GENERIC_REPORT;
                        report[1]=  HID_GENERIC_SET_OVR_MODE & 0xFF;
                        report[2]= mode;
                        for( i=3; i < HID_COMMAND_MAX_SIZE;  i++) report[i]=0;
                        return HID_Write_Data((unsigned char *)report);
        }

        int HID_Set_MultiStream_Mode(int mode)
        {
                unsigned short i;
                char report[HID_COMMAND_MAX_SIZE]={0};
                        report[0] = HID_GENERIC_REPORT;
                        report[1]=  HID_GENERIC_SET_MULTISTREAM_MODE & 0xFF;
                        report[2]= mode;
                        for( i=3; i < HID_COMMAND_MAX_SIZE;  i++) report[i]=0;
                        return HID_Write_Data((unsigned char *)report);
        }

        int HID_Get_MultiStream_Mode()
        {
                int i;
                char report[HID_COMMAND_MAX_SIZE]={0};
                report[0] = HID_GENERIC_REPORT;
                report[1]=  HID_GENERIC_GET_MULTISTREAM_MODE & 0xFF;
                for( i=2; i < HID_COMMAND_MAX_SIZE;  i++) report[i]=0;
                return HID_Write_Data((unsigned char *)report);
        }

        int HID_Set_Phase_Shift(int phase, int channel)
        {
                unsigned short i,tmp_phase = phase;
                char report[HID_COMMAND_MAX_SIZE]={0};
                        report[0] = HID_GENERIC_REPORT;
                        report[1]=  HID_GENERIC_SET_PHASE_SHIFT_COMMAND & 0xFF;
                        report[2]= tmp_phase & 0xFF;
                        report[3]= (tmp_phase >> 8) & 0xFF;
                        report[4]=channel;
                        for( i=5; i < HID_COMMAND_MAX_SIZE;  i++) report[i]=0;
                        return HID_Write_Data((unsigned char *)report);
        }

        int HID_Change_BPF_Tune(unsigned short bpf_value)
        {
                int i;
                char report[HID_COMMAND_MAX_SIZE]={0};
                        report[0] = HID_GENERIC_REPORT;
                        report[1]=  HID_GENERIC_SET_BPF_COMMAND & 0xFF;
                        report[2]= bpf_value & 0xFF;
                        report[3]= (bpf_value >> 8) & 0xFF;
                        for( i=4; i < HID_COMMAND_MAX_SIZE;  i++) report[i]=0;
                        return HID_Write_Data((unsigned char *)report);
        }

        int HID_Set_Multichannel_Mode(int mode, int channel)
        {
                int i;
                char report[HID_COMMAND_MAX_SIZE]={0};
                        report[0] = HID_GENERIC_REPORT;
                        report[1]=  HID_GENERIC_SET_MULTICHANNEL_COMMAND & 0xFF;
                        report[2]= mode & 0xFF;
                        report[3]= channel & 0xFF;
                        for( i=4; i < HID_COMMAND_MAX_SIZE;  i++) report[i]=0;
                        return HID_Write_Data((unsigned char *)report);
        }

        int HID_Save_Default_VCO_voltage(int vco_value)
        {
            int  res;
            long tmp=0;
            tmp = (vco_value + 2048) & 0x0FFF;
            res = HID_Send_EEPROM_Data(VADDRESS_VCO_VOLTAGE, tmp);
            return res;
        }

        int HID_Get_VCO_voltage()
        {
                int i;
                char report[HID_COMMAND_MAX_SIZE]={0};
                report[0] = HID_GENERIC_REPORT;
                report[1]=  HID_GENERIC_GET_VCO_COMMAND & 0xFF;
                report[2]= 0;
                for( i=3; i < HID_COMMAND_MAX_SIZE;  i++) report[i]=0;
                return HID_Write_Data((unsigned char *)report);
        }

        int HID_Get_AGC_State()
        {
                int i;
                char report[HID_COMMAND_MAX_SIZE]={0};
                report[0] = HID_GENERIC_REPORT;
                report[1]=  HID_GENERIC_AGC_GET_STATE_COMMAND & 0xFF;
                for( i=2; i < HID_COMMAND_MAX_SIZE;  i++) report[i]=0;
                return HID_Write_Data((unsigned char *)report);
        }

        int HID_Save_Gain_Table(unsigned short gain_table)
        {
                int i;
                char report[HID_COMMAND_MAX_SIZE]={0};
                        report[0] = HID_GENERIC_REPORT;
                        report[1]=  HID_GENERIC_SAVE_GAIN_TABLE_COMMAND & 0xFF;
                        report[2]= gain_table & 0xFF;
                        report[3]= (gain_table >> 8) & 0xFF;
                        for( i=4; i < HID_COMMAND_MAX_SIZE;  i++) report[i]=0;
                        return HID_Write_Data((unsigned char *)report);
        }

        int HID_Get_Gain_Table()
        {
            int  res=0;
            res = HID_Send_Generic_Command_No_Param(HID_GENERIC_GET_GAIN_TABLE_COMMAND);
            return res;
        }

        int HID_Get_Sync_State()
        {
            int  res=0;
            res = HID_Send_Generic_Command_No_Param(HID_GENERIC_SYNC_WORD_COMMAND);
            return res;
        }

        int HID_Get_HW_CPLD_SN()
        {
            int  res=0;
            res = HID_Send_Generic_Command_No_Param(HID_GENERIC_HW_CPLD_SN_COMMAND);
            return res;
        }
        int HID_Get_CPLD2_SN()
        {
            int  res=0;
            res = HID_Send_Generic_Command_No_Param(HID_GENERIC_HW_CPLD2_SN_COMMAND);
            return res;
        }

        int HID_Check_FW_Version(int value)
        {
                int i;
                char report[HID_COMMAND_MAX_SIZE]={0};
                        report[0] = HID_GENERIC_REPORT;
                        report[1]=  HID_GENERIC_VER_COMMAND & 0xFF;
                        for( i=2; i < HID_COMMAND_MAX_SIZE;  i++) report[i]=0;
                        return HID_Write_Data((unsigned char *)report);
        }

        int HID_console_On(int value)
        {
            int i;
            char report[HID_COMMAND_MAX_SIZE]={0};
            report[0] = HID_GENERIC_REPORT;
            report[1] = HID_GENERIC_HID_CONSOLE_ON & 0xFF;
            report[2] = value;
            for( i=3; i < HID_COMMAND_MAX_SIZE;  i++) report[i]=0;
            return HID_Write_Data((unsigned char *)report);
        }

        int HID_Send_console_char(int value)
        {
            int i;
            char report[HID_COMMAND_MAX_SIZE]={0};
            report[0] = HID_GENERIC_REPORT;
            report[1] = HID_GENERIC_CONSOLE_IO_COMMAND & 0xFF;
            report[2] = value;
            for( i=3; i < HID_COMMAND_MAX_SIZE;  i++) report[i]=0;
            return HID_Write_Data((unsigned char *)report);
        }

        int HID_Init_FE()
	{
                int i;
                char report[HID_COMMAND_MAX_SIZE]={0};
		report[0] = HID_GENERIC_REPORT;
		report[1]=  HID_GENERIC_INIT_FE_COMMAND & 0xFF;
		report[2]=  (~HID_GENERIC_INIT_FE_COMMAND) & 0xFF;
                for( i=3; i < HID_COMMAND_MAX_SIZE;  i++) report[i]=0;
                return HID_Write_Data((unsigned char *)report);
	}

        int HID_Start_UDP_Stream(const unsigned long command)
        {
                int i;
                char report[HID_COMMAND_MAX_SIZE]={0};
				printf("Stream start/stop=> %i\n\r",command);
                report[0] = HID_GENERIC_REPORT;
                report[1]=  HID_GENERIC_START_UDP_STREAM_COMMAND & 0xFF;
                report[2]=  (command) & 0xFF;
                for( i=3; i < HID_COMMAND_MAX_SIZE;  i++) report[i]=0;
                return HID_Write_Data((unsigned char *)report);
        }

        int HID_Set_Broadcast_Flag(const unsigned long command)
        {
                int i;
                char report[HID_COMMAND_MAX_SIZE]={0};
                report[0] = HID_GENERIC_REPORT;
                report[1]=  HID_GENERIC_BROADCAST_COMMAND & 0xFF;
                report[2]=  (command) & 0xFF;
                for( i=3; i < HID_COMMAND_MAX_SIZE;  i++) report[i]=0;
                return HID_Write_Data((unsigned char *)report);
        }

        int HID_Set_Multicast_Flag(const unsigned long command)
        {
                int i;
                char report[HID_COMMAND_MAX_SIZE]={0};
                report[0] = HID_GENERIC_REPORT;
                report[1]=  HID_GENERIC_MULTICAST_COMMAND & 0xFF;
                report[2]=  (command) & 0xFF;
                for( i=3; i < HID_COMMAND_MAX_SIZE;  i++) report[i]=0;
                return HID_Write_Data((unsigned char *)report);
        }

        int HID_Get_Broadcast_Flag()
        {
            int  res=0;
            res = HID_Send_Generic_Command_No_Param(HID_GENERIC_GET_BROADCAST_STATE_COMMAND);
            return res;
        }

        int HID_Send_LPF_Bypass(short command)
        {
            int i;
            char report[HID_COMMAND_MAX_SIZE]={0};
            report[0] = HID_GENERIC_REPORT;
            report[1]=  HID_GENERIC_BYPASS_LPF_COMMAND & 0xFF;
            report[2]=  (command) & 0xFF;
            for( i=3; i < HID_COMMAND_MAX_SIZE;  i++) report[i]=0;
            return HID_Write_Data((unsigned char *)report);
        }


        int HID_Set_LED_On_Off(char mode)
        {
            int i;
            char report[HID_COMMAND_MAX_SIZE]={0};
            report[0] = HID_GENERIC_REPORT;
            report[1]=  HID_SET_LED_ON_OFF & 0xFF;
            report[2]=  (mode);
            for( i=3; i < HID_COMMAND_MAX_SIZE;  i++) report[i]=0;
            return HID_Write_Data((unsigned char *)report);
        }

        int HID_Get_Emulation_ID()
        {
            int  res=0;
            res = HID_Send_Generic_Command_No_Param(HID_GENERIC_GET_AFEDRI_ID_COMMAND);
            return res;
        }

        int HID_Set_ID_String(const unsigned long command)
        {
                int i;
                char report[HID_COMMAND_MAX_SIZE]={0};
                report[0] = HID_GENERIC_REPORT;
                report[1]=  HID_GENERIC_SET_AFEDRI_ID_COMMAND & 0xFF;
                report[2]=  (command) & 0xFF;
                for( i=3; i < HID_COMMAND_MAX_SIZE;  i++) report[i]=0;
                return HID_Write_Data((unsigned char *)report);
        }

        int HID_Save_Default_ID(const unsigned long command)
        {
                int i;
                char report[HID_COMMAND_MAX_SIZE]={0};
                report[0] = HID_GENERIC_REPORT;
                report[1]=  HID_GENERIC_SAVE_AFEDRI_ID_COMMAND & 0xFF;
                report[2]=  (command) & 0xFF;
                for( i=3; i < HID_COMMAND_MAX_SIZE;  i++) report[i]=0;
                return HID_Write_Data((unsigned char *)report);
        }

        int HID_Save_Multistream_mode(unsigned short mode)
        {
                int i;
                char report[HID_COMMAND_MAX_SIZE]={0};
                report[0] = HID_GENERIC_REPORT;
                report[1]=  HID_GENERIC_SAVE_MULTISTREAM_MODE & 0xFF;
                report[2]=  (mode) & 0xFF;
                for( i=3; i < HID_COMMAND_MAX_SIZE;  i++) report[i]=0;
                return HID_Write_Data((unsigned char *)report);
        }

        int HID_Set_DHCP_Server_State(int command)
        {
                int i;
                char report[HID_COMMAND_MAX_SIZE]={0};
                report[0] = HID_GENERIC_REPORT;
                report[1]=  HID_GENERIC_SET_DHCP_STATE_COMMAND & 0xFF;
                report[2]=  (command) & 0xFF;
                for( i=3; i < HID_COMMAND_MAX_SIZE;  i++) report[i]=0;
                return HID_Write_Data((unsigned char *)report);
        }

        int HID_Get_DHCP_Server_State()
        {
            int  res=0;
            res = HID_Send_Generic_Command_No_Param(HID_GENERIC_GET_DHCP_STATE_COMMAND);
            return res;
        }

        long HID_Get_Init_Status()
	{
                int i;
                char report[HID_COMMAND_MAX_SIZE]={0};
		report[0] = HID_GENERIC_REPORT;
		report[1]=  HID_GENERIC_GET_INIT_STATUS_COMMAND & 0xFF;
		report[2]=  (~HID_GENERIC_GET_INIT_STATUS_COMMAND) & 0xFF;
                for( i=3; i < HID_COMMAND_MAX_SIZE;  i++) report[i]=0;
                return HID_Write_Data((unsigned char *)report);
	}

        int HID_Send_defaults_to_EEPROM()
        {
            int i;
            char report[HID_COMMAND_MAX_SIZE]={0};
            report[0] = HID_GENERIC_REPORT;
            report[1]=  HID_GENERIC_SAVE_DEFAULT_PARAMS_COMMAND & 0xFF;
            report[2]= (~HID_GENERIC_SAVE_DEFAULT_PARAMS_COMMAND) & 0xFF;
            for( i=3; i < HID_COMMAND_MAX_SIZE;  i++) report[i]=0;
            return HID_Write_Data((unsigned char *)report);
        }

        int HID_Firmware_Upgrade()
        {
            int  res=0;
            res = HID_Send_Generic_Command_No_Param(HID_GENERIC_IAP_COMMAND);
            return res;
        }

        int HID_SoftReset_SDR()
        {
            int  res=0;
            res = HID_Send_Generic_Command_No_Param(HID_GENERIC_SOFT_RESET_COMMAND);
            return res;
        }

        int HID_Get_AGC_Level(int channel)
        {
            char report[HID_COMMAND_MAX_SIZE]={0};
            report[0] = HID_GENERIC_REPORT;
            report[1]=  HID_GENERIC_AGC_LEVEL_COMMAND & 0xFF;
            report[2]= channel;
            return HID_Write_Data((unsigned char *)report);
        }

        int HID_Get_EEPROM_Data(unsigned char VirtAddress)
        {
//            int i;
            char report[HID_COMMAND_MAX_SIZE]={0};
            report[0] = HID_GENERIC_REPORT;
            report[1]=  HID_READ_EEPROM_COMMAND & 0xFF;
            report[2]= VirtAddress;
            return HID_Write_Data((unsigned char *)report);
        }

        int HID_wait_device_answer()
        {
            int hid_timeout = 0;
            while(hid_command_send_state)
            {
#if defined ( __linux__ ) || defined (__APPLE__)
                sleep(1);
#else
                Sleep(1);
#endif
                if(hid_timeout < 1000L) hid_timeout++;
                else return -1;
            }
            return 0;
        }

        int HID_Get_Main_Clock_Frequency()
        {
            int res=0;
            res = HID_Get_EEPROM_Data(VADDRESS_MAIN_CLOCK_FREQ_LOW_HALFWORD);
            HID_wait_device_answer();
            res = HID_Get_EEPROM_Data(VADDRESS_MAIN_CLOCK_FREQ_HIGH_HALFWORD);
            return res;
        }

        int HID_Get_SDR_IP_Address()
        {
            int  res=0;
            res = HID_Send_Generic_Command_No_Param(HID_GENERIC_GET_SDR_IP_ADDRESS);
            return res;
        }

        int HID_Save_SDR_IP_Address(const unsigned long ip_address)
        {
            int  res;
            res =  HID_Send_Generic_Long_Data(HID_GENERIC_SAVE_SDR_IP_ADDRESS, ip_address);
            return res;
        }

        int HID_Get_SDR_IP_Mask()
        {
            int  res=0;
            res = HID_Send_Generic_Command_No_Param(HID_GENERIC_GET_SDR_IP_MASK);
            return res;
        }

        int HID_Save_SDR_IP_Mask(const unsigned long ip_mask)
        {
            int  res;
            res =  HID_Send_Generic_Long_Data(HID_GENERIC_SAVE_SDR_IP_MASK, ip_mask);
            return res;
        }

        int HID_Get_Gateway_IP_Address()
        {
            int  res=0;
            res = HID_Send_Generic_Command_No_Param(HID_GENERIC_GET_GATEWAY_IP_ADDRESS);
            return res;
        }

        int HID_Save_Gateway_IP_Address(const unsigned long ip_address)
        {
            int  res;
            res =  HID_Send_Generic_Long_Data(HID_GENERIC_SAVE_GATEWAY_IP_ADDRESS, ip_address);
            return res;
        }

        int HID_Get_DST_IP_Address()
        {
            int  res=0;
            res = HID_Send_Generic_Command_No_Param(HID_GENERIC_GET_DST_IP_ADDRESS);
            return res;
        }

        int HID_Save_DST_IP_Address(const unsigned long ip_address)
        {
            int  res;
            res =  HID_Send_Generic_Long_Data(HID_GENERIC_SAVE_DST_IP_ADDRESS, ip_address);
            return res;
        }

        int HID_Get_UDP_Port()
        {
            int  res=0;
            res = HID_Send_Generic_Command_No_Param(HID_GENERIC_GET_DST_PORT);
            return res;
        }

        int HID_Save_UDP_Port(const unsigned long port)
        {
            int  res;
            res =  HID_Send_Generic_Long_Data(HID_GENERIC_SAVE_DST_PORT, port);
            return res;
        }

        int HID_Get_MAC_word()
        {
            int  res=0;
            res = HID_Send_Generic_Command_No_Param(HID_GENERIC_GET_MAC_COMMAND);
            return res;
        }

        int HID_Save_MAC_word(const unsigned long macw)
        {
            int  res;
            res =  HID_Send_Generic_Long_Data(HID_GENERIC_SAVE_MAC_COMMAND, macw);
            return res;
        }

        int HID_Send_Main_Clock_Frequency(unsigned long main_frequency)
        {
            int  res;
            unsigned long tmp=0;
            tmp = main_frequency & 0xFFFF;
            res = HID_Send_EEPROM_Data(VADDRESS_MAIN_CLOCK_FREQ_LOW_HALFWORD, tmp);
            tmp = (main_frequency >> 16) & 0xFFFF;
            HID_wait_device_answer();
            res = HID_Send_EEPROM_Data(VADDRESS_MAIN_CLOCK_FREQ_HIGH_HALFWORD, tmp);
            return res;
        }

        int HID_Send_Sample_Rate(unsigned long SampleRate)
        {
            int  res;
            unsigned long tmp=0;
            tmp = SampleRate & 0xFFFF;
            res = HID_Send_EEPROM_Data(VADDRESS_SAMPLE_RATE_LO, tmp);
//            sleep(100);
            tmp = (SampleRate >> 16) & 0xFFFF;
            HID_wait_device_answer();
            res = HID_Send_EEPROM_Data(VADDRESS_SAMPLE_RATE_HI, tmp);
            return res;
        }

        int HID_Set_New_Sample_Rate(unsigned long SampleRate)
        {
            int  res;
            res =  HID_Send_Generic_Long_Data(HID_GENERIC_SET_SAMPLE_RATE_COMMAND, SampleRate);
            return res;
        }

        int HID_Send_Diversity_Sample_Rate(unsigned long SampleRate)
        {
            int  res;
            unsigned long tmp=0;
            tmp = SampleRate & 0xFFFF;
            res = HID_Send_EEPROM_Data(VADDRESS_DIVERSITY_SAMPLE_RATE_LO, tmp);
            tmp = (SampleRate >> 16) & 0xFFFF;
            HID_wait_device_answer();
            res = HID_Send_EEPROM_Data(VADDRESS_DIVERSITY_SAMPLE_RATE_HI, tmp);
            return res;
        }

        int HID_Send_Diversity_Mode(short mode)
        {
            int  res;
            unsigned long tmp=0;
            tmp = mode & 0xFFFF;
            res = HID_Send_EEPROM_Data(VADDRESS_DIVERSITY_MODE, tmp);
            return res;
        }

        int HID_Send_USB_Mode(short command)
        {
            int i;
            char report[HID_COMMAND_MAX_SIZE]={0};
            report[0] = HID_GENERIC_REPORT;
            report[1]=  HID_GENERIC_SET_USB_MODE & 0xFF;
            report[2]=  (command) & 0xFF;
            for( i=3; i < HID_COMMAND_MAX_SIZE;  i++) report[i]=0;
            return HID_Write_Data((unsigned char *)report);
//            res = HID_Send_EEPROM_Data(HID_GENERIC_SET_USB_MODE, tmp);
//            return res;
        }

        int HID_Get_Sample_Rate()
        {
            int  res=0;
            res = HID_Send_Generic_Command_No_Param(HID_GENERIC_GET_SR_COMMAND);
            return res;
        }

        int HID_Get_Sample_Rate_No_set()
        {
	    int i=0;	
            char report[HID_COMMAND_MAX_SIZE]={0};
            report[0] = HID_GENERIC_REPORT;
            report[1]=  HID_GENERIC_GET_SR_COMMAND & 0xFF;
            report[2]= 0x5A;
            for( i=3; i < HID_COMMAND_MAX_SIZE;  i++) report[i]=0;
            return HID_Write_Data((unsigned char *)report);
        }

        int HID_Get_Diversity_Mode()
        {
            int  res=0;
            res = HID_Send_Generic_Command_No_Param(HID_GENERIC_GET_DIVERSITY_MODE);
            return res;
        }

        int HID_Get_Serial_Number()
        {
            int  res=0;
            res = HID_Send_Generic_Command_No_Param(HID_GENERIC_GET_SERIAL_NUMBER_COMMAND);
            return res;
        }

        int HID_Send_Generic_Command_No_Param(unsigned char command)
        {
            int i;
            char report[HID_COMMAND_MAX_SIZE]={0};
            report[0] = HID_GENERIC_REPORT;
            report[1] = command;
            report[2] = 0;
            for( i=3; i < HID_COMMAND_MAX_SIZE;  i++) report[i]=0;
            return HID_Write_Data((unsigned char *)report);
        }

        int HID_Send_Generic_Long_Data(unsigned char command, unsigned long eeprom_data)
        {
            int i;
            char report[HID_COMMAND_MAX_SIZE]={0};
            report[0] = HID_GENERIC_REPORT;
            report[1]=  command & 0xFF;
            report[2]=  eeprom_data & 0xFF;
            report[3]= (eeprom_data >> 8)& 0xFF;
            report[4]= (eeprom_data >> 16)& 0xFF;
            report[5]= (eeprom_data >> 24)& 0xFF;
            for( i=6; i < HID_COMMAND_MAX_SIZE;  i++) report[i]=0;
            return HID_Write_Data((unsigned char *)report);
        }

        int HID_Send_To_Flash_HalfWord(unsigned short address, unsigned short data)
        {
            int i;
            char report[HID_COMMAND_MAX_SIZE]={0};
            report[0] = HID_GENERIC_REPORT;
            report[1]=  HID_GENERIC_WRITE_HWORD_TO_FLASH;
            report[2]=  address & 0xFF;
            report[3]= (address >> 8)& 0xFF;
            report[4]= data & 0xFF;
            report[5]= (data >> 8)& 0xFF;
            for( i=6; i < HID_COMMAND_MAX_SIZE;  i++) report[i]=0;
            return HID_Write_Data((unsigned char *)report);
        }

        int HID_Send_EEPROM_Data(unsigned char VirtAddress, unsigned short eeprom_data)
        {
            int i;
            char report[HID_COMMAND_MAX_SIZE]={0};
            report[0] = HID_MEMORY_READ_WRITE_REPORT;
            report[1]=  HID_WRITE_EEPROM_COMMAND & 0xFF;
            report[2]= VirtAddress;
            report[3]= eeprom_data & 0xFF;
            report[4]= (eeprom_data >> 8)& 0xFF;
            for( i=5; i < HID_COMMAND_MAX_SIZE;  i++) report[i]=0;
            return HID_Write_Data((unsigned char *)report);
        }

        int HID_Set_R820T_frequency(const unsigned long frequency)
        {
            int  res;
            res =  HID_Send_Generic_Long_Data(HID_GENERIC_SET_R820T_FREQ_COMMAND, frequency);
            return res;
        }

        int HID_Set_R820T_PWR_On_Off(char mode)
        {
            int i;
            char report[HID_COMMAND_MAX_SIZE]={0};
            report[0] = HID_GENERIC_REPORT;
            report[1]=  HID_GENERIC_SET_R820T_PWR_ON_COMMAND;
            report[2]=  mode;
            for( i=3; i < HID_COMMAND_MAX_SIZE;  i++) report[i]=0;
            return HID_Write_Data((unsigned char *)report);
        }

        int HID_Set_R820T_LNA_AGC(char mode)
        {
            int i;
            char report[HID_COMMAND_MAX_SIZE]={0};
            report[0] = HID_GENERIC_REPORT;
            report[1]=  HID_GENERIC_SET_R820T_LNA_AGC_COMMAND;
            report[2]=  mode;
            for( i=3; i < HID_COMMAND_MAX_SIZE;  i++) report[i]=0;
            return HID_Write_Data((unsigned char *)report);
        }

        int HID_Set_R820T_Mixer_AGC(char mode)
        {
            int i;
            char report[HID_COMMAND_MAX_SIZE]={0};
            report[0] = HID_GENERIC_REPORT;
            report[1]=  HID_GENERIC_SET_R820T_MIXER_AGC_COMMAND;
            report[2]=  mode;
            for( i=3; i < HID_COMMAND_MAX_SIZE;  i++) report[i]=0;
            return HID_Write_Data((unsigned char *)report);
        }

        int HID_Set_R820T_VGA_Gain(unsigned char gain)
        {
            int i;
            char report[HID_COMMAND_MAX_SIZE]={0};
            report[0] = HID_GENERIC_REPORT;
            report[1]=  HID_GENERIC_SET_R820T_VGA_GAIN_COMMAND;
            report[2]=  gain;
            for( i=3; i < HID_COMMAND_MAX_SIZE;  i++) report[i]=0;
            return HID_Write_Data((unsigned char *)report);
        }

        int HID_Set_R820T_LNA_Gain(unsigned char gain)
        {
            int i;
            char report[HID_COMMAND_MAX_SIZE]={0};
            report[0] = HID_GENERIC_REPORT;
            report[1]=  HID_GENERIC_SET_R820T_LNA_GAIN_COMMAND;
            report[2]=  gain;
            for( i=3; i < HID_COMMAND_MAX_SIZE;  i++) report[i]=0;
            return HID_Write_Data((unsigned char *)report);
        }

        int HID_Set_R820T_MIX_Gain(unsigned char gain)
        {
            int i;
            char report[HID_COMMAND_MAX_SIZE]={0};
            report[0] = HID_GENERIC_REPORT;
            report[1]=  HID_GENERIC_SET_R820T_MIXER_GAIN_COMMAND;
            report[2]=  gain;
            for( i=3; i < HID_COMMAND_MAX_SIZE;  i++) report[i]=0;
            return HID_Write_Data((unsigned char *)report);
        }

        int HID_Set_R820T_IF_BW(unsigned char bw)
        {
            int i;
            char report[HID_COMMAND_MAX_SIZE]={0};
            report[0] = HID_GENERIC_REPORT;
            report[1]=  HID_GENERIC_SET_R820T_IF_BW_COMMAND;
            report[2]=  bw;
            for( i=3; i < HID_COMMAND_MAX_SIZE;  i++) report[i]=0;
            return HID_Write_Data((unsigned char *)report);
        }//HID_Set_R820T_IF_BW

        int HID_Set_R820T_HP_BW(unsigned char bw)
        {
            int i;
            char report[HID_COMMAND_MAX_SIZE]={0};
            report[0] = HID_GENERIC_REPORT;
            report[1]=  HID_GENERIC_SET_R820T_HIGHPASS_BW_COMMAND;
            report[2]=  bw;
            for( i=3; i < HID_COMMAND_MAX_SIZE;  i++) report[i]=0;
            return HID_Write_Data((unsigned char *)report);
        }//HID_Set_R820T_IF_BW

        int HID_Set_R820T_LP_BW(unsigned char bw)
        {
            int i;
            char report[HID_COMMAND_MAX_SIZE]={0};
            report[0] = HID_GENERIC_REPORT;
            report[1]=  HID_GENERIC_SET_R820T_LOWPASS_BW_COMMAND;
            report[2]=  bw;
            for( i=3; i < HID_COMMAND_MAX_SIZE;  i++) report[i]=0;
            return HID_Write_Data((unsigned char *)report);
        }//HID_Set_R820T_IF_BW

        int  HID_Save_VHF_Reference_Clock(int Reference_clock)
        {
            int  res;
            res =  HID_Send_Generic_Long_Data(HID_GENERIC_SAVE_R820T_REF_FREQ_COMMAND, Reference_clock);
            return res;
        }
        int HID_Get_VHF_Reference_Clock()
        {
            int  res=0;
            res = HID_Send_Generic_Command_No_Param(HID_GENERIC_GET_R820T_REF_FREQ_COMMAND);
            return res;
        }

double RF_Gain_Int_2_Double(int value)
{
    double val = (double) value;
    double Vmag = 0.25;//0.0488;//(Volts)
    double Vdbs =0;
    double Gbn, Gt;
    Vdbs = 1.50*val/255.0;
    Gbn = 20 * Vdbs / 0.6;//dB
    Gt = Gbn + 20 * log10(Vmag/0.5);
    return Gt;
}


int RF_Gain_Calc(int value)
{
    int rf_gain=0;
    rf_gain = RF_GAIN_ZERO + 3*(value>>3);
    return rf_gain;
}

int RF_Gain_IntDb_2_Int(int gain)
{
    double val = 0;
    double Vmag = 0.25;//0.0488;//(Volts)
    double Vdbs =0;
    double Gbn, Gt;
    int result = 0;
    Gt = (double)gain;
    Gbn = Gt - 20 * log10(Vmag/0.5);
    Vdbs = Gbn * 0.6 / 20;
    val = Vdbs * 255.0 / 1.5;
    result = val;
    return result;
}

//	int HID_Check_Device_Status()

//End of File
