/*********************************************************************
*	AFEDRI SDR-Net Commander utility
*	Author: Alexander Trushkin ( 4Z5LV )
*	Date: 9.10.2013
*	
*	This utility allows you to control AFEDRI SDR 
*	over USB connection.
*	So you can use this utility in command shell scripts or WebSDR
*	or in crone command   
**********************************************************************/

#ifndef _HELP_HEADER_
	char *help_text ="\
	Please use next commands with AFEDRi SDR:\n\r\
	-F <frequency>\n\r\
	 : set the new central reception frequency of SDR \n\r\
		example: \n\r\
		# sdr_commander -F14100000 \n\r\
	-R   \n\r\
	 : reset SDR, this command makes soft reset of SDR \n\r\
	   example: \n\r\
		# sdr_commander -R \n\r\
	-H      \n\r\
	 : help, this command shows this help \n\r\
	   example: \n\r\
		# sdr_commander -H \n\r\
	-V      \n\r\
	 : verbose, this command shows the main AFEDRI SDR  parameters that stored in EEPROM \n\r\
	   example: \n\r\
	   # sdr_commander -V \n\r\
	-D      \n\r\
	 : save defaults, this command saves the current SDR parameters to EEPROM,  \n\r\
	   so after SDR reboot it will start by default with this parameters. \n\r\
	   The next parameters are stored in SDR's EEPROM: \n\r\
	   RF Gain, FE Gain, Central SDR frequency \n\r\
	   example: \n\r\
		# sdr_commander -D \n\r\
	-G <RF_gain>\n\r\
	 : set the new RF Gain of SDR in dB units\n\r\
		example: \n\r\
		# sdr_commander -G14 \n\r\
		where -G can be integer number from -10 (dB) to 35 (dB) \n\r\
	-I <RF_gain_index>\n\r\
	 : with number only (i.e. -i7)  - sets the new RF Gain of SDR in indexed units\n\r\
	 : with l letter (i.e. -il7) - sets the new R820T LNA Gain of SDR in indexed units\n\r\
	 : with m letter (i.e. -im7) - sets the new R820T Mixer Gain of SDR in indexed units\n\r\
		example: \n\r\
		# sdr_commander -I1 \n\r\
		where -I can be integer number from 0 ( -10dB gain) to 15 ( 35dB gain) \n\r\
	-E <FE_gain_index>\n\r\
	 : set the new FE Gain of SDR in indexed units\n\r\
		example: \n\r\
		# sdr_commander -E1 \n\r\
		where -E can be integer number from 0 ( x1 gain) to 6 ( x4 gain) \n\r\
	-S <sample_rate_frequency>\n\r\
	 : set the new USB sample rate  of AFEDRI SDR ,\n\r\
	 Can be used with additional key going right after S, like D or N, \n\r\
	   the SDR will be automatically rebooted in the end of this command\n\r\
		example: \n\r\
		# sdr_commander -S192000 // - Saves the sample rate for Single Channel modes\n\r\
		# sdr_commander -SD96000 // - Saves the sample rate for Dual Channel modes\n\r\
		# sdr_commander -SN96000 // - Set the new sample rate immediately - do not save it into EEPROM and not reboot\n\r\
	-U <usb_mode>\n\r\
	 : save the new USB mode\n\r\
		example: \n\r\
		# sdr_commander -U1 \n\r\
		where -U argument can be integer number: \n\r\
		0 ( MSD mode ) or\n\r\
		1 ( USB mode ) or\n\r\
		2 (OpenHPSDR mode - will reset SDR automatically)\n\r\
	-L \n\r\
	 : Print list of USB HID devices that was found on this PC \n\r\
		example: \n\r\
		# sdr_commander -L \n\r\
	-C \n\r\
	 : Set SDR channel (For Multichannel SDR) for each the new setting will be applied \n\r\
		used with other keys like -f or -g: \n\r\
		example: \n\r\
		# sdr_commander -C2 -f14100000 //sets frequency\n\r\
		14100000 for channel 2\n\r\
	-A \n\r\
	 : Set desired AGC mode (for Multichannel SDR) the bit the state \n\r\
	 	of AGC for each channel corresponds to bit state  \n\r\
	 : a with l letter -al0 - disbale/enable the R820T LNA AGC\n\r\
	 : a with m letter -am0 - disbale/enable the R820T Mixer AGC\n\r\
		example: \n\r\
		# sdr_commander -A3 //enable AGC for channels 1 and 2, disable for all other channels\n\r\
	-q \n\r\
	 : Set multichannel SDR's mode  \n\r\
	   The possible mode settings:  \n\r\
	   0 - Single channel mode  \n\r\
	   1 - Diversity (Dual channel) mode  \n\r\
	   2 - Dual channel (independent) mode  \n\r\
	   3 - Diversity (Single channel) mode    \n\r\
	   4 - Diversity (Quad channel) mode  \n\r\
	   5 - Quad channel (independent) mode  \n\r\
		example: \n\r\
		# sdr_commander -q2 //sets SDR to mode 2 (Dual Channel mode)\n\r\
	-P <Path to MSD device or TCP port number>\n\r\
	 : set the path to AFEDRI MSD device or TCP port number when using netwokr mode (-T flag defined),\n\r\
		(MSD) example Windows: \n\r\
		# sdr_commander -PG:\ \n\r\
		(MSD) example  Linux:\n\r\
		# sdr_commander -P/run/media/alex/disk/ \n\r\
		see network example in -t key description \n\r\
		The commands will be executed from left to right \n\r\
	-T < TCP IP address>\n\r\
	 : set TCP IP address of AFEDRI SDR-Net device,\n\r\
		# sdr_commander -t192.168.0.8 -p50000 \n\r\
		The commands will be executed from left to right \n\r\
	-N <usb_serial_number>\n\r\
	 : set the serial numebr of AFEDRI SDR USB device,\n\r\
		example Windows: \n\r\
		# sdr_commander -N00011234500  \n\r\
	-X <data stream state>\n\r\
	 : enable (x=1) or disable (x=0) datastream output from SDR  device,\n\r\
		example: \n\r\
		# sdr_commander -x1  \n\r\
	-J \n\r\
	 : Saves or Reads the two last bytes of network interface MAC address \n\r\
	 : j with ?  -j? - prints the current value of MAC address\n\r\
	 : j with with HEX number -j0x3A34 - saves desired (user defined) value of the MAC address\n\r\
		example: \n\r\
		# sdr_commander -j0x3A34 //save the new value 0x3A34 of last bytes of MAC \n\r\
/*********************************************************************************/\n\r\
	The most commands excluding -S and -R can be combined in one string, see example below: \n\r\
	# sdr_commander -F14100000 -G14 -E0 \n\r\
	This command will do next: \n\r\
    1. Sets AFEDRI's Central (LO) frequency to 14100000 Hz \n\r\
    2. Sets AFEDRI's RF Gain to 14dB \n\r\
    3. Sets AFEDRI's FE Gain to x1\n\r\
";
#endif
//End if header
